#ifndef GFXGL_UTILS_INCLUDED // -*- C++ -*-
#define GFXGL_UTILS_INCLUDED

/*************************************************************************
 *
 * Here are some more generally useful things.
 *
 *************************************************************************/

inline void glOffsetForMesh(float factor=1.0, float bias=1e-6)
{
#ifdef GL_POLYGON_OFFSET_EXT
    glEnable(GL_POLYGON_OFFSET_EXT);
    glPolygonOffsetEXT(factor, bias);
#endif
}

inline void glNoOffset()
{
#ifdef GL_POLYGON_OFFSET_EXT
    glDisable(GL_POLYGON_OFFSET_EXT);
#endif
}

#ifdef GFXMATH_MAT4_INCLUDED
inline void glGetMatrix(Mat4& m, GLenum name=GL_MODELVIEW_MATRIX)
{
    Mat4 tmp;

    glGetDoublev(name, &tmp(0,0));
    //
    // glMatrix layout is column major
    m = tmp.transpose();
}

inline void glLoadMatrix(const Mat4& m)
{
    Mat4 tmp = m.transpose();
    glLoadMatrixd(&tmp(0,0));
}

inline void glMultMatrix(const Mat4& m)
{
    Mat4 tmp = m.transpose();
    glMultMatrixd(&tmp(0,0));
}
#endif

inline void glGetViewport(int *x, int *y, int *w, int *h)
{
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);

    if( x ) *x = viewport[0];
    if( y ) *y = viewport[1];
    if( w ) *w = viewport[2];
    if( h ) *h = viewport[3];
}

inline void glUnproject(int win_x, int win_y, int win_z,
			double *x, double *y, double *z)
{
    GLdouble modelMatrix[16];
    GLdouble projMatrix[16];
    GLint viewport[4];

    glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
    glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
    glGetIntegerv(GL_VIEWPORT, viewport);

    gluUnProject(win_x,win_y,win_z,
		 modelMatrix, projMatrix, viewport,
		 x, y, z);
}

typedef union {
    unsigned int pixel;
    struct { unsigned char r,g,b,a; } channel;
} gfxPixel;


inline gfxPixel *glSnapshot(int x, int y, int w, int h)
{
    gfxPixel *data = new gfxPixel[w*h];

    glReadBuffer(GL_FRONT);
    glReadPixels(x,y,w,h,
		 GL_RGBA,
		 GL_UNSIGNED_BYTE,
		 data);

    return data;
}

#ifdef __STDIO_H__
inline void glToPPM(FILE *out)
{
    int x,y,w,h;
    
    glGetViewport(&x, &y, &w, &h);
    gfxPixel *data = glSnapshot(x,y,w,h);

    fprintf(out,"P6 %u %u 255\n",w,h);
    int i,j;
    for(j=h-1;j>=0;j--)
	for(i=0;i<w;i++)
	{
	    int index = j*w + i;
	    fputc(data[index].channel.r, out);
	    fputc(data[index].channel.g, out);
	    fputc(data[index].channel.b, out);
	}

    delete[] data;
}
#endif

inline ostream& operator<<(ostream& out,const gfxPixel& p)
{
    return out << p.channel.r << p.channel.g << p.channel.b;
}

inline void glToPPM(ostream& out)
{
    int x,y,w,h;
    
    glGetViewport(&x, &y, &w, &h);
    gfxPixel *data = glSnapshot(x,y,w,h);

    out << "P6 " << w <<" "<< h << " 255" << endl;
    int i,j;
    for(j=h-1;j>=0;j--)
	for(i=0;i<w;i++)
	{
	    int index = j*w + i;
	    out << data[index];
	}

    delete[] data;
}


// GFXGL_UTILS_INCLUDED
#endif
