/***** Ball.h *****/
#ifndef _H_Ball
#define _H_Ball
#include "BallAux.h"

typedef enum AxisSet{NoAxes, CameraAxes, BodyAxes, OtherAxes, NSets} AxisSet;
typedef enum BallType { ArcBall, TrackBall } BallType;
typedef float *ConstraintSet;
typedef struct {
    HVect center;
    double radius, aspect;
    Quat qNow, qDown, qDrag;
    HVect vNow, vDown, vFrom, vTo, vrFrom, vrTo;
    HMatrix mNow, mDown;
    int showResult, dragging;
    ConstraintSet sets[NSets];
    int setSizes[NSets];
    AxisSet axisSet;
    int axisIndex;
    BallType ballType;
} BallData;

/* Public routines */
extern BallData *Ball_Alloc();
extern void Ball_Free(BallData *);
extern void Ball_Init(BallData *ball, double aspect);
extern void Ball_Place(BallData *ball, HVect center, double radius
		       /*, Quat orient*/);
extern void Ball_Mouse(BallData *ball, HVect vNow);
extern void Ball_UseSet(BallData *ball, AxisSet axisSet);
extern void Ball_ShowResult(BallData *ball);
extern void Ball_HideResult(BallData *ball);
extern void Ball_Update(BallData *ball, int al);
extern void Ball_Value(BallData *ball, HMatrix mNow);
extern void Ball_BeginDrag(BallData *ball);
extern void Ball_EndDrag(BallData *ball);
extern void Ball_Draw(BallData *ball, int drawBall,
		      int drawConstraints, int drawArc);
extern int Ball_MouseOutsideSphere(BallData *ball, HVect vMouse);
extern void Ball_SetBallType(BallData *ball, BallType bt);
/* Private routines */
void DrawAnyArc(HVect vFrom, HVect vTo);
void DrawHalfArc(HVect n);
void Ball_DrawConstraints(BallData *ball);
void Ball_DrawDragArc(BallData *ball);
void Ball_DrawResultArc(BallData *ball);
#endif
