#include <gfx/std.h>
#include <gfx/sys/timing.h>

#if defined(WIN32)
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

// NOTE:  A more intelligent way to do this would be to track the
// 64-bit time counter and do arithmetic on that rather than
// converting to doubles (and hence losing precision)
//
// WARNING: This is actually measuring REAL TIME, not process
// time as the Unix version does.  GetThreadTimes() would give
// us access to user process time, but it's only available in NT!
double gfx_get_time()
{
	// Find out what time it is now
	SYSTEMTIME now;
	GetSystemTime(&now);

	// Flatten into 100ns clicks
	FILETIME flat;
	SystemTimeToFileTime(&now, &flat);

	// Convert into something we can do math on
	LARGE_INTEGER i;
	i.LowPart = flat.dwLowDateTime;
	i.HighPart = flat.dwHighDateTime;

	// Convert to seconds and return
	return (double)(i.QuadPart) / 1e7;	
}

#elif defined(GFX_NO_RUSAGE)
#include <limits.h>
#include <sys/times.h>

double gfx_get_time()
{
    struct tms t;

    times(&t);

    return (double)(t.tms_utime) / (double)CLK_TCK;
}

#else
#include <sys/time.h>
#include <sys/resource.h>

double gfx_get_time()
{
    struct rusage t;

    getrusage(RUSAGE_SELF, &t);

    return (double)t.ru_utime.tv_sec + (double)t.ru_utime.tv_usec/1000000;
}
#endif
