// $Id: qslim.cxx,v 1.3 1997/10/01 14:07:27 garland Exp $

#include "qslim.h"
#include "decimate.h"

#include <gfx/sys/timing.h>


static int init_argc;
static char **init_argv;

#ifdef WIN32
static inline int gethostname(char *str, int N)
{ strncpy(str, "Win32-box", N); return 0; }
#else
#  include <unistd.h>
#endif
#include <time.h>
void out_annotate(ostream& out, Model&)
{
    out << "# Generated";
    char buf[66];
    time_t curtime = time(NULL);

    if( !gethostname(buf, 64) )
    {
	out << " on " << buf;
    }

    strftime(buf, 64, "%I:%M:%S %p %Z on %d %b %Y", localtime(&curtime));
    out << " at " << buf << endl;

    out << "# cmd: ";
    for(int i=0; i<init_argc; i++)
	out << " " << init_argv[i];
    out << endl;
}


void out_annotate(ostream& out, Vertex *v)
{
    if( selected_output & OUTPUT_VERT_NOTES )
	out << "#$vid " << v->uniqID+1 << endl;

    if( selected_output & OUTPUT_QUADRICS )
    {
	Mat4 Q;
	if( decimate_quadric(v, Q) )
	{
	    out << "#$Q";
	    for(int row=0; row<4; row++)
		for(int col=0; col<4; col++)
		    out << " " << Q[row][col];
	    out << endl;
	}		
    }

    if( selected_output & OUTPUT_COST )
    {
	Mat4 Q;
	if( decimate_quadric(v, Q) )
	{
	    out << "#$err " << (*v * (Q * *v)) << endl;
	}		
    }


}

void out_annotate(ostream& out, Face *t)
{
    if( selected_output & OUTPUT_FACE_NOTES )
	out << "#$fid " << t->uniqID+1 << endl;
}

int initialVertCount;
int initialEdgeCount;
int initialFaceCount;

double setup_time, init_time, slim_time, write_time;


static
void qslim_init(int argc, char **argv)
{
    int i;

    setup_time = gfx_get_time();

    cerr << "Reading input ..." << endl;
    process_cmdline(argc, argv);

    initialVertCount = M0.vertCount();
    initialEdgeCount = M0.edgeCount();
    initialFaceCount = M0.faceCount();

    cerr << "Cleaning up initial input ..." << endl;
    //
    // Get rid of degenerate faces
    for(i=0; i<M0.faceCount(); i++)
	if( !M0.face(i)->plane().isValid() )
	    M0.killFace(M0.face(i));

    M0.removeDegeneracy(M0.allFaces());
    //
    // Get rid of unused vertices
    for(i=0; i<M0.vertCount(); i++)
    {
	if( M0.vertex(i)->edgeUses().length() == 0 )
	    M0.vertex(i)->kill();
    }

    cerr << "Input model summary:" << endl;
    cerr << "    Vertices    : " << initialVertCount << endl;
    cerr << "    Edges       : " << initialEdgeCount << endl;

    int man=0, non=0, bndry=0, bogus=0;
    for(i=0; i<M0.edgeCount(); i++)
        switch( M0.edge(i)->faceUses().length() )
        {
        case 0:
            bogus++;
            break;
        case 1:
            bndry++;
            break;
        case 2:
            man++;
            break;
        default:
            non++;
            break;
        }
    if( bogus )
        cerr << "        Bogus       : " << bogus << endl;
    cerr << "        Boundary    : " << bndry << endl;
    cerr << "        Manifold    : " << man << endl;
    cerr << "        Nonmanifold : " << non << endl;

    cerr << "    Faces       : " << initialFaceCount << endl;

    cerr << "    BBox        : "
         << M0.bounds.min << ":" << M0.bounds.max << endl;
    cerr << "    BSphere     : "
	 << M0.bounds.center << ":" << M0.bounds.radius << endl;

    setup_time = gfx_get_time() - setup_time;
}

static
void qslim_run()
{
    init_time = gfx_get_time();
    decimate_init(M0, pair_selection_tolerance);
    init_time = gfx_get_time() - init_time;


    slim_time = gfx_get_time();
    while( M0.validFaceCount > face_target
	   && decimate_min_error() < error_tolerance )
	decimate_contract(M0);
    slim_time = gfx_get_time() - slim_time;

    if( outfile )
    {
        write_time = gfx_get_time();
        *outfile << M0;
        write_time = gfx_get_time() - write_time;
    }
    else
        write_time = 0.0;


    cerr << endl << endl;
    cerr << "Setup time        : " << setup_time << " sec" << endl;
    cerr << "Quadrix init time : " << init_time << " sec" << endl;
    cerr << "Quadrix slim time : " << slim_time << " sec" << endl;
    cerr << "Output time       : " << write_time << " sec" << endl;
    cerr << endl;
    cerr << "Total time        : " << setup_time+init_time+slim_time+write_time
         << " sec" << endl;
}

main(int argc, char **argv)
{
    init_argc = argc;
    init_argv = argv;

#ifdef GUI
    gui_init(&argc, argv);
#endif

    qslim_init(argc, argv);

#ifdef GUI
    gui_loop();
#else
    qslim_run();
#endif

    return 0;
}
